// Kinetophone_narrator_model.hpp
//
// Copyright 2011-2012 Roan Trail, Inc.
//
// This file is part of Kinetophone.
//
// Kinetophone is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 2 of the License,
// or (at your option) any later version.
//
// Kinetophone is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.  You should have received
// a copy of the GNU General Public License along with Kinetophone. If
// not, see <http://www.gnu.org/licenses/>.

#ifndef KINETOPHONE_NARRATOR_MODEL_HPP_
#define KINETOPHONE_NARRATOR_MODEL_HPP_

#include <string>

namespace Gdk
{
  class Pixbuf;
}

namespace Glib
{
  template <class T> class RefPtr;
}

using std::string;
using Gdk::Pixbuf;
using Glib::RefPtr;

namespace Roan_trail
{
  class Error_param;

  namespace Recorder
  {
    class Narration_recorder;
  }
  namespace Source
  {
    template <class Image> class Slide_collection;
  }

  using Roan_trail::Recorder::Narration_recorder;
  using Roan_trail::Source::Slide_collection;

  namespace Kinetophone
  {
    class Kinetophone_slide_collection;

    class Kinetophone_narrator_model
    {
    public:
      // constructor/destructor
      Kinetophone_narrator_model(const string& slides_source_type);
      ~Kinetophone_narrator_model();
      // accessors
      const Slide_collection<RefPtr<Pixbuf> >* slides() const { return m_slides; }
      Slide_collection<RefPtr<Pixbuf> >* slides() { return m_slides; }
      const Narration_recorder* narration_recorder() const { return m_recorder; }
      Narration_recorder* narration_recorder() { return m_recorder; }
      bool is_dirty() const { return m_dirty; }
      // mutators
      void set_dirty(bool dirty = true) { m_dirty = dirty; }
      // commands
      bool import_from_XML(const string& file_path, Error_param& return_error);
      bool export_as_XML(const string& file_path, Error_param& return_error) const;
    protected:
      // invariant check
      bool mf_invariant(bool check_base_class = true) const;
    private:
      Slide_collection<RefPtr<Pixbuf> >* m_slides;
      Narration_recorder* m_recorder;
      bool m_dirty;
      // prevent compiler from generating
      Kinetophone_narrator_model(const Kinetophone_narrator_model& narrator_model);
      Kinetophone_narrator_model& operator=(const Kinetophone_narrator_model& narrator_model);
    };
  }
}

#endif // KINETOPHONE_NARRATOR_MODEL_HPP_
